/*
 * deviced
 *
 * Copyright (c) 2015 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <stdio.h>
#include <libsyscommon/dbus-system.h>

#include "device-interface.h"
#include "util.h"
#include "core.h"

#define ENLIGHTENMENT_BUS_NAME          "org.enlightenment.wm"
#define ENLIGHTENMENT_OBJECT_PATH       "/org/enlightenment/wm"
#define ENLIGHTENMENT_INTERFACE_NAME    ENLIGHTENMENT_BUS_NAME".dpms"

static void dpms_set_cb(GVariant *var, void *user_data, GError *err)
{
	int state;

	if (!var) {
		_E("no message [%s], Cannot get pmlock check",
		    err->message);
		return;
	}

	if (!dh_get_param_from_var(var, "(i)", &state)) {
		_E("no message [%s]", g_variant_get_type_string(var));
		goto out;
	}

	_D("DPMS was set %d", state);

#ifdef ENABLE_PM_LOG
	switch (state) {
	case DPMS_ON:
		pm_history_save(PM_LOG_LCD_ON, pm_cur_state);
		break;
	case DPMS_OFF:
		pm_history_save(PM_LOG_LCD_OFF, pm_cur_state);
		break;
	default:
		break;
	}
#endif

out:
	g_variant_unref(var);
}

int dpms_set_power(enum dpms_state state)
{
	int ret;

	ret = dbus_handle_method_async_with_reply_var(ENLIGHTENMENT_BUS_NAME,
			ENLIGHTENMENT_OBJECT_PATH,
			ENLIGHTENMENT_INTERFACE_NAME,
			"set",
			g_variant_new("(u)", state),
			dpms_set_cb,
			-1,
			NULL);

	if (ret < 0)
		return ret;

	return 0;
}

int dpms_get_power(enum dpms_state *state)
{
	int ret;

	if (!state)
		return -EINVAL;

	ret = dbus_handle_method_sync(ENLIGHTENMENT_BUS_NAME,
			ENLIGHTENMENT_OBJECT_PATH,
			ENLIGHTENMENT_INTERFACE_NAME,
			"get",
			NULL, NULL);
	if (ret < 0)
		return ret;

	*state = ret;
	return 0;
}
