/*
 * deviced
 *
 * Copyright (c) 2016 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#include <stdio.h>
#include <errno.h>
#include <libsyscommon/dbus-system.h>

#include "common.h"
#include "devices.h"
#include "device-notifier.h"
#include "log.h"

static device_notifier_state_e state = DEVICE_NOTIFIER_STATE_STOP;

GVariant *dbus_start_event_handler(GDBusConnection *conn,
	const gchar *sender, const gchar *path, const gchar *iface, const gchar *name,
	GVariant *param, GDBusMethodInvocation *invocation, gpointer user_data)
{
	if (state != DEVICE_NOTIFIER_STATE_START) {
		_D("Start event-handler.");
		state = DEVICE_NOTIFIER_STATE_START;
		device_notify(DEVICE_NOTIFIER_EVENT_HANDLER, (void *)&state);
	}
	return g_variant_new("(i)", 0);
}

GVariant *dbus_stop_event_handler(GDBusConnection *conn,
	const gchar *sender, const gchar *path, const gchar *iface, const gchar *name,
	GVariant *param, GDBusMethodInvocation *invocation, gpointer user_data)
{
	if (state != DEVICE_NOTIFIER_STATE_STOP) {
		_D("Stop event-handler.");
		state = DEVICE_NOTIFIER_STATE_STOP;
		device_notify(DEVICE_NOTIFIER_EVENT_HANDLER, (void *)&state);
	}
	return g_variant_new("(i)", 0);
}

GVariant *dbus_udev_event_handler(GDBusConnection *conn,
	const gchar *sender, const gchar *path, const gchar *iface, const gchar *name,
	GVariant *param, GDBusMethodInvocation *invocation, gpointer user_data)
{
	char *argv;
	int argc;
	GVariant *ret = NULL;

	g_variant_get(param, "(sis)", NULL, &argc, &argv);

	if (strncmp(argv, "start", strlen("start")) == 0)
		ret = dbus_start_event_handler(conn, sender, path, iface, name, param, invocation, user_data);
	else if (strncmp(argv, "stop", strlen("stop")) == 0)
		ret = dbus_stop_event_handler(conn, sender, path, iface, name, param, invocation, user_data);
	else
		ret = g_variant_new("(i)", -EINVAL);

	g_free(argv);
	return ret;
}

static const dbus_method_s eh_methods[] = {
	{ "EventStart",  NULL,  "i", dbus_start_event_handler },
	{ "EventStop",   NULL,  "i", dbus_stop_event_handler  },
	{ "udev",       "sis",  "i", dbus_udev_event_handler  }, /* deprecated */
	/* Add methods here */
};

static const dbus_interface_u eh_interface = {
	.oh = NULL,
	.name = DEVICED_INTERFACE_SYSNOTI,
	.methods = eh_methods,
	.nr_methods = ARRAY_SIZE(eh_methods),
};

static void event_handler_init(void *data)
{
	int ret;

	ret = dbus_handle_add_dbus_object(NULL, DEVICED_PATH_SYSNOTI, &eh_interface);
	if (ret < 0)
		_E("Failed to init dbus method: %d", ret);

	state = DEVICE_NOTIFIER_STATE_START;
}

static const struct device_ops event_handler_device_ops = {
	.name     = "event-handler",
	.init     = event_handler_init,
};

DEVICE_OPS_REGISTER(&event_handler_device_ops)
